@@ -4,7 +4,7 @@
  * URL: http://libharu.org
  *
  * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
- * Copyright (c) 2007-2008 Antony Dovgal <tony@daylessday.org>
+ * Copyright (c) 2007-2009 Antony Dovgal <tony@daylessday.org>
  *
  * Permission to use, copy, modify, distribute and sell this software
  * and its documentation for any purpose is hereby granted without fee,
@@ -1112,7 +1112,7 @@ ParseCMAP_format4  (HPDF_FontDef  fontdef,
     } else
         attr->cmap.glyph_id_array = NULL;
 
-#ifdef HPDF_DEBUG
+#ifdef LIBHPDF_DEBUG
     /* print all elements of cmap table */
     for (i = 0; i < attr->cmap.seg_count_x2 / 2; i++) {
         HPDF_PTRACE((" ParseCMAP_format4[%d] start_count=0x%04X, "
@@ -1225,7 +1225,7 @@ CheckCompositGryph  (HPDF_FontDef   fontdef,
 {
     HPDF_TTFontDefAttr attr = (HPDF_TTFontDefAttr)fontdef->attr;
     HPDF_UINT offset = attr->glyph_tbl.offsets[gid];
-   // HPDF_UINT len = attr->glyph_tbl.offsets[gid + 1] - offset;
+    /* HPDF_UINT len = attr->glyph_tbl.offsets[gid + 1] - offset; */
     HPDF_STATUS ret;
 
     HPDF_PTRACE ((" CheckCompositGryph\n"));
@@ -1456,7 +1456,7 @@ ParseLoca  (HPDF_FontDef  fontdef)
     }
 
 
-#ifdef HPDF_DEBUG
+#ifdef LIBHPDF_DEBUG
     poffset = attr->glyph_tbl.offsets;
     for (i = 0; i <= attr->num_glyphs; i++) {
         HPDF_PTRACE((" ParseLOCA offset[%u]=%u\n", i, (HPDF_UINT)*poffset));
