@@ -4,7 +4,7 @@
  * URL: http://libharu.org
  *
  * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
- * Copyright (c) 2007-2008 Antony Dovgal <tony@daylessday.org>
+ * Copyright (c) 2007-2009 Antony Dovgal <tony@daylessday.org>
  *
  * Permission to use, copy, modify, distribute and sell this software
  * and its documentation for any purpose is hereby granted without fee,
@@ -221,6 +221,36 @@ HPDF_Image_LoadJpegImage  (HPDF_MMgr        mmgr,
     return image;
 }
 
+HPDF_Image
+HPDF_Image_LoadJpegImageFromMem  (HPDF_MMgr    mmgr,
+                            const HPDF_BYTE   *buf,
+                                  HPDF_UINT    size,
+                                  HPDF_Xref    xref)
+{
+	HPDF_Stream jpeg_data;
+	HPDF_Image image;
+
+	HPDF_PTRACE ((" HPDF_Image_LoadJpegImageFromMem\n"));
+
+	jpeg_data = HPDF_MemStream_New(mmgr,size);
+	if (!HPDF_Stream_Validate (jpeg_data)) {
+		HPDF_RaiseError (mmgr->error, HPDF_INVALID_STREAM, 0);
+		return NULL;
+	}
+
+	if (HPDF_Stream_Write (jpeg_data, buf, size) != HPDF_OK) {
+		HPDF_Stream_Free (jpeg_data);
+		return NULL;
+	}
+
+	image = HPDF_Image_LoadJpegImage(mmgr,jpeg_data,xref);
+
+	/* destroy file stream */
+	HPDF_Stream_Free (jpeg_data);
+
+	return image;
+}
+
 
 HPDF_Image
 HPDF_Image_LoadRawImage (HPDF_MMgr          mmgr,
@@ -304,7 +334,8 @@ HPDF_Image_LoadRawImageFromMem  (HPDF_MMgr          mmgr,
     HPDF_PTRACE ((" HPDF_Image_LoadRawImageFromMem\n"));
 
     if (color_space != HPDF_CS_DEVICE_GRAY &&
-            color_space != HPDF_CS_DEVICE_RGB) {
+            color_space != HPDF_CS_DEVICE_RGB &&
+            color_space != HPDF_CS_DEVICE_CMYK) {
         HPDF_SetError (mmgr->error, HPDF_INVALID_COLOR_SPACE, 0);
         return NULL;
     }
@@ -325,13 +356,21 @@ HPDF_Image_LoadRawImageFromMem  (HPDF_MMgr          mmgr,
     if (ret != HPDF_OK)
         return NULL;
 
-    if (color_space == HPDF_CS_DEVICE_GRAY) {
-        size = (HPDF_DOUBLE)width * height / (8 / bits_per_component) + 0.876;
-        ret = HPDF_Dict_AddName (image, "ColorSpace", COL_GRAY);
-    } else {
-        size = (HPDF_DOUBLE)width * height / (8 / bits_per_component) + 0.876;
-        size *= 3;
-        ret = HPDF_Dict_AddName (image, "ColorSpace", COL_RGB);
+    switch (color_space) {
+        case HPDF_CS_DEVICE_GRAY:
+            size = (HPDF_DOUBLE)width * height / (8 / bits_per_component) + 0.876;
+            ret = HPDF_Dict_AddName (image, "ColorSpace", COL_GRAY);
+            break;
+        case HPDF_CS_DEVICE_RGB:
+            size = (HPDF_DOUBLE)width * height / (8 / bits_per_component) + 0.876;
+            size *= 3;
+            ret = HPDF_Dict_AddName (image, "ColorSpace", COL_RGB);
+            break;
+        case HPDF_CS_DEVICE_CMYK:
+            size = (HPDF_DOUBLE)width * height / (8 / bits_per_component) + 0.876;
+            size *= 4;
+            ret = HPDF_Dict_AddName (image, "ColorSpace", COL_CMYK);
+            break;
     }
 
     if (ret != HPDF_OK)
@@ -579,4 +618,46 @@ HPDF_Image_SetColorMask (HPDF_Image   image,
     return HPDF_OK;
 }
 
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_Image_AddSMask  (HPDF_Image  image,
+                      HPDF_Image  smask)
+{
+
+   const char *name;
+
+   if (!HPDF_Image_Validate (image))
+       return HPDF_INVALID_IMAGE;
+   if (!HPDF_Image_Validate (smask))
+       return HPDF_INVALID_IMAGE;
+
+   if (HPDF_Dict_GetItem (image, "SMask", HPDF_OCLASS_BOOLEAN))
+       return HPDF_RaiseError (image->error, HPDF_INVALID_OPERATION, 0);
+
+   name = HPDF_Image_GetColorSpace (smask);
+   if (!name || HPDF_StrCmp (COL_GRAY, name) != 0)
+       return HPDF_RaiseError (smask->error, HPDF_INVALID_COLOR_SPACE, 0);
+
+   return HPDF_Dict_Add (image, "SMask", smask);
+}
+
+HPDF_STATUS
+HPDF_Image_SetColorSpace  (HPDF_Image   image,
+                          HPDF_Array   colorspace)
+{
+    if (!HPDF_Image_Validate (image))
+        return HPDF_INVALID_IMAGE;
+
+    return HPDF_Dict_Add (image, "ColorSpace", colorspace);
+}
+
+
+HPDF_STATUS
+HPDF_Image_SetRenderingIntent  (HPDF_Image   image,
+                          const char* intent)
+{
+    if (!HPDF_Image_Validate (image))
+        return HPDF_INVALID_IMAGE;
+
+    return HPDF_Dict_AddName (image, "Intent", intent);
+}
 
