@@ -4,7 +4,7 @@
  * URL: http://libharu.org
  *
  * Copyright (c) 1999-2006 Takeshi Kanno <takeshi_kanno@est.hi-ho.ne.jp>
- * Copyright (c) 2007-2008 Antony Dovgal <tony@daylessday.org>
+ * Copyright (c) 2007-2009 Antony Dovgal <tony@daylessday.org>
  *
  * Permission to use, copy, modify, distribute and sell this software
  * and its documentation for any purpose is hereby granted without fee,
@@ -20,6 +20,7 @@
 #include "hpdf.h"
 #include "hpdf_annotation.h"
 #include "hpdf_destination.h"
+#include "hpdf_u3d.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -668,7 +669,7 @@ AddAnnotation  (HPDF_Page        page,
                 HPDF_Annotation  annot)
 {
     HPDF_Array array;
-    HPDF_STATUS ret;
+    HPDF_STATUS ret = HPDF_OK;
 
     HPDF_PTRACE((" HPDF_Pages\n"));
 
@@ -684,8 +685,11 @@ AddAnnotation  (HPDF_Page        page,
         if (ret != HPDF_OK)
             return ret;
     }
-
-    return HPDF_Array_Add (array, annot);
+    
+    ret += HPDF_Array_Add (array, annot);
+    /* Add Parent to the annotation  */
+    ret += HPDF_Dict_Add( annot, "P", page);
+    return ret;
 }
 
 
@@ -974,7 +978,7 @@ HPDF_Page_GetTextRenderingMode  (HPDF_Page   page)
 HPDF_EXPORT(HPDF_REAL)
 HPDF_Page_GetTextRaise  (HPDF_Page   page)
 {
-	return HPDF_Page_GetTextRise (page);
+    return HPDF_Page_GetTextRise (page);
 }
 
 HPDF_EXPORT(HPDF_REAL)
@@ -1262,7 +1266,6 @@ HPDF_Page_SetBoxValue (HPDF_Page          page,
     return HPDF_OK;
 }
 
-
 HPDF_EXPORT(HPDF_STATUS)
 HPDF_Page_SetRotate (HPDF_Page      page,
                      HPDF_UINT16    angle)
@@ -1289,6 +1292,25 @@ HPDF_Page_SetRotate (HPDF_Page      page,
     return ret;
 }
 
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_Page_SetZoom  (HPDF_Page   page,
+                    HPDF_REAL   zoom)
+{
+    HPDF_STATUS ret = HPDF_OK;
+
+    HPDF_PTRACE((" HPDF_Page_SetZoom\n"));
+
+    if (!HPDF_Page_Validate (page)) {
+        return HPDF_INVALID_PAGE;
+    }
+
+    if (zoom < 0.08 || zoom > 32) {
+        return HPDF_RaiseError (page->error, HPDF_INVALID_PARAMETER, 0);
+    }
+
+    ret = HPDF_Dict_AddReal (page, "PZ", zoom);
+    return ret;
+}
 
 HPDF_EXPORT(HPDF_STATUS)
 HPDF_Page_SetWidth  (HPDF_Page    page,
@@ -1399,29 +1421,29 @@ HPDF_Page_CreateDestination  (HPDF_Page   page)
 
 HPDF_EXPORT(HPDF_Annotation)
 HPDF_Page_Create3DAnnot    (HPDF_Page       page,
-							HPDF_Rect       rect,
-							HPDF_U3D u3d)
+                            HPDF_Rect       rect,
+                            HPDF_U3D u3d)
 {
-	HPDF_PageAttr attr;
-	HPDF_Annotation annot;
+    HPDF_PageAttr attr;
+    HPDF_Annotation annot;
 
-	HPDF_PTRACE((" HPDF_Page_Create3DAnnot\n"));
+    HPDF_PTRACE((" HPDF_Page_Create3DAnnot\n"));
 
-	if (!HPDF_Page_Validate (page))
-		return NULL;
+    if (!HPDF_Page_Validate (page))
+        return NULL;
 
-	attr = (HPDF_PageAttr)page->attr;
+    attr = (HPDF_PageAttr)page->attr;
 
-	annot = HPDF_3DAnnot_New (page->mmgr, attr->xref, rect, u3d);
-	if (annot) {
-		if (AddAnnotation (page, annot) != HPDF_OK) {
-			HPDF_CheckError (page->error);
-			annot = NULL;
-		}
-	} else
-		HPDF_CheckError (page->error);
+    annot = HPDF_3DAnnot_New (page->mmgr, attr->xref, rect, u3d);
+    if (annot) {
+        if (AddAnnotation (page, annot) != HPDF_OK) {
+            HPDF_CheckError (page->error);
+            annot = NULL;
+        }
+    } else
+        HPDF_CheckError (page->error);
 
-	return annot;
+    return annot;
 }
 
 HPDF_EXPORT(HPDF_Annotation)
@@ -1445,7 +1467,7 @@ HPDF_Page_CreateTextAnnot  (HPDF_Page          page,
         return NULL;
     }
 
-    annot = HPDF_TextAnnot_New (page->mmgr, attr->xref, rect, text, encoder);
+    annot = HPDF_MarkupAnnot_New (page->mmgr, attr->xref, rect, text, encoder, HPDF_ANNOT_TEXT_NOTES);
     if (annot) {
         if (AddAnnotation (page, annot) != HPDF_OK) {
             HPDF_CheckError (page->error);
@@ -1457,6 +1479,71 @@ HPDF_Page_CreateTextAnnot  (HPDF_Page          page,
     return annot;
 }
 
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateFreeTextAnnot  (HPDF_Page          page,
+                                HPDF_Rect          rect,
+                                const char   *text,
+                                HPDF_Encoder       encoder)
+{
+    HPDF_PageAttr attr;
+    HPDF_Annotation annot;
+
+    HPDF_PTRACE((" HPDF_Page_CreateFreeTextAnnot\n"));
+
+    if (!HPDF_Page_Validate (page))
+        return NULL;
+
+    attr = (HPDF_PageAttr)page->attr;
+
+    if (encoder && !HPDF_Encoder_Validate (encoder)) {
+        HPDF_RaiseError (page->error, HPDF_INVALID_ENCODER, 0);
+        return NULL;
+    }
+
+    annot = HPDF_MarkupAnnot_New (page->mmgr, attr->xref, rect, text, encoder, HPDF_ANNOT_FREE_TEXT);
+    if (annot) {
+        if (AddAnnotation (page, annot) != HPDF_OK) {
+            HPDF_CheckError (page->error);
+            annot = NULL;
+        }
+    } else
+        HPDF_CheckError (page->error);
+
+    return annot;
+}
+
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateLineAnnot  (HPDF_Page          page,
+                            const char           *text,
+                            HPDF_Encoder       encoder)
+{
+    HPDF_PageAttr attr;
+    HPDF_Annotation annot;
+    HPDF_Rect rect = {0,0,0,0};
+
+    HPDF_PTRACE((" HPDF_Page_CreateLineAnnot\n"));
+
+    if (!HPDF_Page_Validate (page))
+        return NULL;
+
+    attr = (HPDF_PageAttr)page->attr;
+
+    if (encoder && !HPDF_Encoder_Validate (encoder)) {
+        HPDF_RaiseError (page->error, HPDF_INVALID_ENCODER, 0);
+        return NULL;
+    }
+
+    annot = HPDF_MarkupAnnot_New (page->mmgr, attr->xref, rect, text, encoder, HPDF_ANNOT_LINE);
+    if (annot) {
+        if (AddAnnotation (page, annot) != HPDF_OK) {
+            HPDF_CheckError (page->error);
+            annot = NULL;
+        }
+    } else
+        HPDF_CheckError (page->error);
+
+    return annot;
+}
 
 HPDF_EXPORT(HPDF_Annotation)
 HPDF_Page_CreateLinkAnnot  (HPDF_Page          page,
@@ -1523,6 +1610,229 @@ HPDF_Page_CreateURILinkAnnot  (HPDF_Page          page,
     return annot;
 }
 
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateCircleAnnot (HPDF_Page          page,
+                             HPDF_Rect          rect,
+                             const char            *text,
+                             HPDF_Encoder       encoder)
+{
+    HPDF_PageAttr attr;
+    HPDF_Annotation annot;
+
+    HPDF_PTRACE((" HPDF_Page_CreateCircleAnnot\n"));
+
+    if (!HPDF_Page_Validate (page))
+        return NULL;
+
+    attr = (HPDF_PageAttr)page->attr;
+
+    if (encoder && !HPDF_Encoder_Validate (encoder)) {
+        HPDF_RaiseError (page->error, HPDF_INVALID_ENCODER, 0);
+        return NULL;
+    }
+
+    annot = HPDF_MarkupAnnot_New (page->mmgr, attr->xref, rect, text, encoder, HPDF_ANNOT_CIRCLE);
+    if (annot) {
+        if (AddAnnotation (page, annot) != HPDF_OK) {
+            HPDF_CheckError (page->error);
+            annot = NULL;
+        }
+    } else
+        HPDF_CheckError (page->error);
+
+    return annot;
+}
+
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateSquareAnnot (HPDF_Page          page,
+                             HPDF_Rect          rect,
+                             const char            *text,
+                             HPDF_Encoder       encoder)
+{
+    HPDF_PageAttr attr;
+    HPDF_Annotation annot;
+
+    HPDF_PTRACE((" HPDF_Page_CreateCircleAnnot\n"));
+
+    if (!HPDF_Page_Validate (page))
+        return NULL;
+
+    attr = (HPDF_PageAttr)page->attr;
+
+    if (encoder && !HPDF_Encoder_Validate (encoder)) {
+        HPDF_RaiseError (page->error, HPDF_INVALID_ENCODER, 0);
+        return NULL;
+    }
+
+    annot = HPDF_MarkupAnnot_New (page->mmgr, attr->xref, rect, text, encoder, HPDF_ANNOT_SQUARE);
+    if (annot) {
+        if (AddAnnotation (page, annot) != HPDF_OK) {
+            HPDF_CheckError (page->error);
+            annot = NULL;
+        }
+    } else
+        HPDF_CheckError (page->error);
+
+    return annot;
+}
+
+HPDF_EXPORT(HPDF_Dict)
+HPDF_Page_Create3DView    (HPDF_Page       page,
+                           HPDF_U3D        u3d,
+                           HPDF_Annotation    annot3d,
+                           const char *name)
+{
+    HPDF_PageAttr attr;
+    HPDF_Dict view;
+
+    HPDF_PTRACE((" HPDF_Page_Create3DView\n"));
+
+    if (!HPDF_Page_Validate (page))
+        return NULL;
+
+    attr = (HPDF_PageAttr)page->attr;
+
+    view = HPDF_3DView_New( page->mmgr, attr->xref, u3d, name);
+    if (!view) {
+        HPDF_CheckError (page->error);
+    }
+    return view;
+}
+
+HPDF_Annotation
+HPDF_Page_CreateTextMarkupAnnot (HPDF_Page     page,
+                                HPDF_Rect      rect,
+                                const char     *text,
+                                HPDF_Encoder   encoder,
+                                HPDF_AnnotType subType)
+{
+    HPDF_PageAttr attr;
+    HPDF_Annotation annot;
+
+    HPDF_PTRACE((" HPDF_Page_CreateTextMarkupAnnot\n"));
+
+    if (!HPDF_Page_Validate (page))
+        return NULL;
+
+    attr = (HPDF_PageAttr)page->attr;
+
+    if (encoder && !HPDF_Encoder_Validate (encoder)) {
+        HPDF_RaiseError (page->error, HPDF_INVALID_ENCODER, 0);
+        return NULL;
+    }
+
+    annot = HPDF_MarkupAnnot_New ( page->mmgr, attr->xref, rect, text, encoder, subType);
+    if (annot) {
+        if (AddAnnotation (page, annot) != HPDF_OK) {
+            HPDF_CheckError (page->error);
+            annot = NULL;
+        }
+    } else
+        HPDF_CheckError (page->error);
+
+    return annot;
+}
+
+
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateHighlightAnnot  (HPDF_Page          page,
+                                HPDF_Rect          rect,
+                                const char   *text,
+                                HPDF_Encoder       encoder)
+{
+    HPDF_PTRACE((" HPDF_Page_CreateHighlightAnnot\n"));
+
+    return HPDF_Page_CreateTextMarkupAnnot( page, rect, text, encoder, HPDF_ANNOT_HIGHTLIGHT);
+}
+
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateSquigglyAnnot  (HPDF_Page          page,
+                                HPDF_Rect          rect,
+                                const char   *text,
+                                HPDF_Encoder       encoder)
+{
+    HPDF_PTRACE((" HPDF_Page_CreateSquigglyAnnot\n"));
+
+    return HPDF_Page_CreateTextMarkupAnnot( page, rect, text, encoder, HPDF_ANNOT_SQUIGGLY);
+}
+
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateUnderlineAnnot  (HPDF_Page          page,
+                                HPDF_Rect          rect,
+                                const char   *text,
+                                HPDF_Encoder       encoder)
+{
+    HPDF_PTRACE((" HPDF_Page_CreateUnderlineAnnot\n"));
+
+    return HPDF_Page_CreateTextMarkupAnnot( page, rect, text, encoder, HPDF_ANNOT_UNDERLINE);
+}
+
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateStrikeOutAnnot  (HPDF_Page          page,
+                                HPDF_Rect          rect,
+                                const char   *text,
+                                HPDF_Encoder       encoder)
+{
+    HPDF_PTRACE((" HPDF_Page_CreateStrikeOutAnnot\n"));
+
+    return HPDF_Page_CreateTextMarkupAnnot( page, rect, text, encoder, HPDF_ANNOT_STRIKE_OUT);
+}
+
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreatePopupAnnot  (    HPDF_Page          page,
+                                HPDF_Rect          rect,
+                                HPDF_Annotation       parent)
+{
+    HPDF_PageAttr attr;
+    HPDF_Annotation annot;
+
+    HPDF_PTRACE((" HPDF_Page_CreatePopupAnnot\n"));
+
+    if (!HPDF_Page_Validate (page))
+        return NULL;
+
+    attr = (HPDF_PageAttr)page->attr;
+
+    annot = HPDF_PopupAnnot_New ( page->mmgr, attr->xref, rect, parent);
+    if (annot) {
+        if (AddAnnotation (page, annot) != HPDF_OK) {
+            HPDF_CheckError (page->error);
+            annot = NULL;
+        }
+    } else
+        HPDF_CheckError (page->error);
+
+    return annot;
+}
+
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateStampAnnot  (    HPDF_Page           page,
+                                HPDF_Rect           rect,
+                                HPDF_StampAnnotName name,
+                                const char*            text,
+                                HPDF_Encoder        encoder)
+{
+    HPDF_PageAttr attr;
+    HPDF_Annotation annot;
+
+    HPDF_PTRACE((" HPDF_Page_CreateStampAnnot\n"));
+
+    if (!HPDF_Page_Validate (page))
+        return NULL;
+
+    attr = (HPDF_PageAttr)page->attr;
+
+    annot = HPDF_StampAnnot_New ( page->mmgr, attr->xref, rect, name, text, encoder);
+    if (annot) {
+        if (AddAnnotation (page, annot) != HPDF_OK) {
+            HPDF_CheckError (page->error);
+            annot = NULL;
+        }
+    } else
+        HPDF_CheckError (page->error);
+
+    return annot;
+}
 
 void
 HPDF_Page_SetFilter  (HPDF_Page    page,
